﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Sidebar
{
    /// <summary>
    /// Interaktionslogik für AddAppWindow.xaml
    /// </summary>
    public partial class AddAppWindow : Window
    {
        public AddAppWindow()
        {
            InitializeComponent();
        }

        public static bool Open(bool edit = false, ApplicationUserItem app = null)
        {
            if(edit)
            {
                AddAppWindow win = new AddAppWindow();

                win.Topmost = true;

                win.Title = "Edit shortcut";

                win.txtExePath.Text = app.Path;
                win.txtArguments.Text = app.Arguments;
                win.txtName.Text = app.Name;
                win.chkExtract.IsChecked = app.UseEmbeddedIcon;

                if(win.ShowDialog().Value)
                {
                    int index = Config.Applications.IndexOf(app);

                    app.Arguments = win.txtArguments.Text;
                    app.Name = win.txtName.Text;
                    app.Path = win.txtExePath.Text;
                    app.UseEmbeddedIcon = win.chkExtract.IsChecked.Value;

                    Config.Applications[index] = app;

                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                AddAppWindow win = new AddAppWindow();

                win.Title = "Create shortcut";

                if (win.ShowDialog().Value)
                {
                    app = new ApplicationUserItem() ;

                    app.Arguments = win.txtArguments.Text;
                    app.Name = win.txtName.Text;
                    app.Path = win.txtExePath.Text;
                    app.UseEmbeddedIcon = win.chkExtract.IsChecked.Value;

                    Config.Applications.Add(app);

                    return true;
                }
                else
                {
                    return false;
                }
            }
        }

        private void AutoDetect()
        {
            if (File.Exists(txtExePath.Text))
            {
                if (ShortcutReader.IsShortcut(txtExePath.Text))
                {
                    Shortcut s = ShortcutReader.GetShortcutInfo(txtExePath.Text);

                    txtArguments.Text = s.Arguments;
                    txtExePath.Text = s.Path;
                    txtName.Text = s.Name;
                }
                else
                {
                    txtName.Text = System.IO.Path.GetFileNameWithoutExtension(txtExePath.Text);
                }

                chkExtract.IsChecked = true;
            }
        }

        private void grpDrop_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop)) e.Effects = DragDropEffects.Link;
        }

        private void grpDrop_Drop(object sender, DragEventArgs e)
        {
            string[] files = (string[]) e.Data.GetData(DataFormats.FileDrop);

            if(files.Length > 1)
            {
                MessageBox.Show("Please drop only one file.", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            else
            {
                txtExePath.Text = files[0];

                this.AutoDetect();
            }
        }

        private void btnClose_Click(object sender, RoutedEventArgs e)
        {
            this.DialogResult = false;
            this.Close();
        }

        private void btnSave_Click(object sender, RoutedEventArgs e)
        {
            if(File.Exists(txtExePath.Text) && !string.IsNullOrWhiteSpace(txtName.Text))
            {
                this.DialogResult = true;
                this.Close();
            }
            else
            {
                MessageBox.Show("Please enter correct data!", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnBrowse_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "Executables|*.exe";

            if(ofd.ShowDialog().Value)
            {
                txtExePath.Text = ofd.FileName;
                this.AutoDetect();
            }
        }
    }
}
