﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Sidebar
{
    /// <summary>
    /// Interaktionslogik für UserControl1.xaml
    /// </summary>
    public partial class ArrowButton : UserControl
    {
        public enum Direction
        {
            Top = -90,
            Right = 0,
            Bottom = 90,
            Left = 180,
            Undefined = -1
        }

        private bool useAnimations = true;

        public event EventHandler<MouseButtonEventArgs> Clicked;

        public ArrowButton()
        {
            InitializeComponent();

            ArrowDirection = Direction.Bottom;
        }

        public bool UseAnimations
        {
            get
            {
                return useAnimations;
            }

            set
            {
                useAnimations = value;
            }
        }

        public Direction ArrowDirection
        {
            get
            {
                switch((int) arrowRotation.Angle)
                {
                    case 90:
                        return Direction.Bottom;
                    case 0:
                        return Direction.Right;
                    case -90:
                        return Direction.Top;
                    case 180:
                        return Direction.Left;
                }

                return Direction.Undefined;
            }

            set
            {
                if(value != Direction.Undefined)
                {
                    if(useAnimations)
                    {
                        DoubleAnimation anim = new DoubleAnimation(arrowRotation.Angle, (int) value, new Duration(new TimeSpan(0, 0, 0, 0, 300)), FillBehavior.HoldEnd);
                        arrowRotation.BeginAnimation(RotateTransform.AngleProperty, anim);
                    }
                    else
                    {
                        arrowRotation.Angle = (int) value;
                    }
                }
            }
        }

        private void grid_MouseUp(object sender, MouseButtonEventArgs e)
        {
            Clicked?.Invoke(this, e);
        }
    }
}
