﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media.Imaging;

namespace Sidebar
{
    public class BitmapImageConverter
    {
        public static BitmapImage BitmapToImageSource(Bitmap bitmap)
        {
            using (MemoryStream memory = new MemoryStream())
            {
                bitmap.Save(memory, ImageFormat.Png);
                memory.Position = 0;
                BitmapImage bitmapimage = new BitmapImage();
                bitmapimage.BeginInit();
                bitmapimage.StreamSource = memory;
                bitmapimage.CacheOption = BitmapCacheOption.OnLoad;
                bitmapimage.EndInit();

                return bitmapimage;
            }
        }

        [DllImport("gdi32.dll", SetLastError = true)]
        static extern bool DeleteObject(IntPtr hObject);

        [DllImport("user32.dll")]
        static extern bool GetIconInfo(IntPtr hIcon, out ICONINFO piconinfo);

        public struct ICONINFO
        {
            public bool fIcon;
            public int xHotspot;
            public int yHotspot;
            public IntPtr hbmMask;
            public IntPtr hbmColor;
        }
        
        public static Bitmap IconToAlphaBitmap(Icon ico)
        {
            ICONINFO ii = new ICONINFO();
            GetIconInfo(ico.Handle, out ii);
            Bitmap bmp = Bitmap.FromHbitmap(ii.hbmColor);
            DeleteObject(ii.hbmColor);
            DeleteObject(ii.hbmMask);

            if (Bitmap.GetPixelFormatSize(bmp.PixelFormat) < 32)
                return ico.ToBitmap();

            BitmapData bmData;
            Rectangle bmBounds = new Rectangle(0, 0, bmp.Width, bmp.Height);

            bmData = bmp.LockBits(bmBounds, ImageLockMode.ReadOnly, bmp.PixelFormat);

            Bitmap dstBitmap = new Bitmap(bmData.Width, bmData.Height, bmData.Stride, PixelFormat.Format32bppArgb, bmData.Scan0);

            bool IsAlphaBitmap = false;

            for (int y = 0; y <= bmData.Height - 1; y++)
            {
                for (int x = 0; x <= bmData.Width - 1; x++)
                {
                    Color PixelColor = Color.FromArgb(Marshal.ReadInt32(bmData.Scan0, (bmData.Stride * y) + (4 * x)));
                    if (PixelColor.A > 0 & PixelColor.A < 255)
                    {
                        IsAlphaBitmap = true;
                        break;
                    }
                }
                if (IsAlphaBitmap) break;
            }

            bmp.UnlockBits(bmData);

            if (IsAlphaBitmap == true)
                return new Bitmap(dstBitmap);
            else
                return new Bitmap(ico.ToBitmap());

        }

        public static BitmapImage IconToBitmapImage(Icon icon)
        {
            return BitmapToImageSource(IconToAlphaBitmap(icon));
        }
    }
}
