﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Sidebar
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private WindowState state;
        private KeyboardHook hook = new KeyboardHook();
        private Dictionary<ApplicationUserItem, ImageSource> apps = new Dictionary<ApplicationUserItem, ImageSource>();

        public MainWindow()
        {
            InitializeComponent();
        }

        #region Window Positioning
        /// <summary>
        /// Only top position yet
        /// </summary>

        public void ToggleWindow(bool maximize)
        {
            Rect target = GetTargetRectangle(maximize);

            //{ BeginTime = new TimeSpan(0, 0, 0, 0, Config.AnimationSpeed) }
            //{ BeginTime = new TimeSpan(0) }

            //Animate window size
            DoubleAnimation animHeight = new DoubleAnimation(this.Height, target.Height, new Duration(new TimeSpan(0, 0, 0, 0, Config.AnimationSpeed)), FillBehavior.HoldEnd) { BeginTime = new TimeSpan(0, 0, 0, 0, Config.AnimationSpeed) };
            DoubleAnimation animWidth = new DoubleAnimation(this.Width, target.Width, new Duration(new TimeSpan(0, 0, 0, 0, Config.AnimationSpeed)), FillBehavior.HoldEnd) { BeginTime = new TimeSpan(0) };

            //Animate window location
            DoubleAnimation animX = new DoubleAnimation(this.Left, target.X, new Duration(new TimeSpan(0, 0, 0, 0, Config.AnimationSpeed)), FillBehavior.HoldEnd) { BeginTime = new TimeSpan(0) };
            DoubleAnimation animY = new DoubleAnimation(this.Top, target.Y, new Duration(new TimeSpan(0, 0, 0, 0, Config.AnimationSpeed)), FillBehavior.HoldEnd) { BeginTime = new TimeSpan(0, 0, 0, 0, Config.AnimationSpeed) };

            //Create storyboard
            Storyboard s = new Storyboard();

            s.Duration = new Duration(new TimeSpan(0, 0, 0, 0, Config.AnimationSpeed * 2));

            s.Children.Add(animHeight);
            s.Children.Add(animWidth);
            s.Children.Add(animX);
            s.Children.Add(animY);

            Storyboard.SetTarget(animX, this);
            Storyboard.SetTarget(animY, this);
            Storyboard.SetTarget(animHeight, this);
            Storyboard.SetTarget(animWidth, this);

            Storyboard.SetTargetProperty(animX, new PropertyPath(Window.LeftProperty));
            Storyboard.SetTargetProperty(animY, new PropertyPath(Window.TopProperty));
            Storyboard.SetTargetProperty(animHeight, new PropertyPath(Window.HeightProperty));
            Storyboard.SetTargetProperty(animWidth, new PropertyPath(Window.WidthProperty));

            //Toggle arrow
            s.Completed += (object sender, EventArgs e) =>
            {
                arrowButton.ArrowDirection = GetTargetArrowDirection(maximize);
            };

            //Animate
            s.Begin();

            //Status
            if (maximize) state = WindowState.Maximized;
            else state = WindowState.Minimized;
        }

        private Rect GetTargetRectangle(bool maximized)
        {
            if (maximized)
            {
                Rect r = new Rect(GetTargetSize(true));

                r.X = this.Left - (r.Width - this.Width) / 2;
                r.Y = 0;

                return r;
            }
            else
            {
                Rect r = new Rect(GetTargetSize(false));

                r.X = this.Left + (this.Width - r.Width) / 2;
                r.Y = 0;

                return r;
            }
        }

        public Size GetTargetSize(bool maximized)
        {
            if (maximized)
            {
                //Later calculate with col/row
                return new Size(1200, 500);
            }
            else
            {
                return new Size(130, 15);
            }
        }

        public ArrowButton.Direction GetTargetArrowDirection(bool maximized)
        {
            if (maximized)
            {
                return ArrowButton.Direction.Top;
            }
            else
            {
                return ArrowButton.Direction.Bottom;
            }
        }

        public Point GetCenteredPosition(bool maximized)
        {
            return new Point(SystemParameters.PrimaryScreenWidth / 2 - GetTargetSize(maximized).Width / 2, 0);
        }
        #endregion

        #region Initialization
        private void Main_Loaded(object sender, RoutedEventArgs e)
        {
            editMode = false;

            this.InitializePosition();

            this.ReloadApps();

            this.DisplayApps();
            
            hook.KeyDown += Hook_KeyDown;
            hook.KeyUp += Hook_KeyUp;

            this.ReloadHotkeys();
        }

        private void InitializePosition()
        {
            this.Opacity = 0.2;

            this.Width = GetTargetSize(false).Width;
            this.Height = GetTargetSize(false).Height;

            this.Left = SystemParameters.PrimaryScreenWidth / 2 - GetTargetSize(false).Width / 2;
            this.Top = 0;

            ToggleWindow(false);
        }
        #endregion

        #region Edit Mode
        private bool editMode;

        public void EditMode(bool enter)
        {
            if (enter)
            {
                editMode = true;

                edit.Visibility = Visibility.Hidden;
                check.Visibility = Visibility.Visible;

                foreach (AppIcon a in appList.Children)
                {
                    a.ShowBin = true;
                }
            }
            else
            {
                edit.Visibility = Visibility.Visible;
                check.Visibility = Visibility.Hidden;
                
                foreach (AppIcon a in appList.Children)
                {
                    a.ShowBin = false;
                }

                ReloadApps();

                this.DisplayApps();

                editMode = false;

                Config.SaveConfig();
            }
        }
        #endregion

        #region Interaction
        private void arrowButton_Clicked(object sender, MouseButtonEventArgs e)
        {
            if (editMode)
            {
                EditMode(false);
            }

            //Toggle according to state
            ToggleWindow(state != WindowState.Maximized);
        }

        private void edit_Clicked(object sender, MouseEventArgs e)
        {
            EditMode(true);
        }

        private void check_Clicked(object sender, MouseEventArgs e)
        {
            if (check.Visibility == Visibility.Visible && editMode)
            {
                EditMode(false);
            }
        }

        private void add_Clicked(object sender, MouseEventArgs e)
        {
            if (editMode)
            {
                EditMode(false);
            }

            this.Topmost = false;

            if (AddAppWindow.Open())
            {
                ReloadApps();

                this.DisplayApps();
            }

            this.Topmost = true;
        }

        private void settings_Clicked(object sender, MouseEventArgs e)
        {
            if (editMode)
            {
                EditMode(false);
            }

            this.Topmost = false;

            new SettingsWindow().ShowDialog();

            this.Topmost = true;

            this.ReloadHotkeys();
        }

        private void exit_Clicked(object sender, MouseEventArgs e)
        {
            if(editMode)
            {
                EditMode(false);
            }

            Config.SaveConfig();

            Application.Current.Shutdown(0);
        }

        private void info_Clicked(object sender, MouseEventArgs e)
        {
            this.Topmost = false;

            new InfoWindow().Show();

            this.Topmost = true;
        }
        #endregion

        #region Application List Management
        private void ReloadApps()
        {
            apps.Clear();

            List<ApplicationUserItem> applist = Config.Applications;

            foreach (ApplicationUserItem app in applist)
            {
                apps.Add(app, app.UseEmbeddedIcon ? app.GetImage() : null);
            }
        }

        private void DisplayApps()
        {
            waitingIcon.Visibility = Visibility.Visible;
            appList.Visibility = Visibility.Hidden;

            appList.Children.Clear();

            foreach (var app in apps)
            {
                AppIcon ic = new AppIcon()
                {
                    ConnectedApp = app.Key,
                    Text = app.Key.Name
                };

                if (app.Value != null)
                {
                    ic.Icon = app.Value;
                }

                if (editMode)
                {
                    ic.ShowBin = true;
                }

                ic.Clicked += Ic_Clicked;
                ic.BinClicked += Ic_BinClicked;

                appList.Children.Add(ic);
            }

            waitingIcon.Visibility = Visibility.Hidden;
            appList.Visibility = Visibility.Visible;
        }

        private void Ic_BinClicked(object sender, MouseEventArgs e)
        {
            AppIcon a = sender as AppIcon;

            Config.Applications.Remove(a.ConnectedApp);

            appList.Children.Remove(a);
        }

        private void Ic_Clicked(object sender, MouseEventArgs e)
        {
            AppIcon a = sender as AppIcon;

            if (editMode)
            {
                this.Topmost = false;

                if (AddAppWindow.Open(true, a.ConnectedApp))
                {
                    this.ReloadApps();

                    this.DisplayApps();
                }

                this.Topmost = true;
            }
            else
            {
                if (File.Exists(a.ConnectedApp.Path))
                {
                    Process.Start(a.ConnectedApp.Path, a.ConnectedApp.Arguments);

                    if (Config.CollapseAfterRun)
                    {
                        ToggleWindow(false);
                    }
                }
                else
                {
                    MessageBox.Show("The executable does not exist anymore.", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }

        private void HideApps()
        {
            appList.Children.Clear();
        }
        #endregion

        #region Hotkey
        private bool ctrlPressed = false;
        private bool shiftPressed = false;
        private bool keyPressed = false;

        private void ReloadHotkeys()
        {
            if(Config.UseHotKey)
            {
                hook.Uninstall();

                ctrlPressed = false;
                shiftPressed = false;
                keyPressed = false;

                hook.Install();
            }
            else
            {
                hook.Uninstall();
            }
        }

        private void Hook_KeyUp(KeyboardHook.VKeys key)
        {
            if (key == KeyboardHook.VKeys.LSHIFT || key == KeyboardHook.VKeys.RSHIFT || key == KeyboardHook.VKeys.SHIFT)
            {
                shiftPressed = false;
            }
            else if (key == KeyboardHook.VKeys.LCONTROL || key == KeyboardHook.VKeys.RCONTROL || key == KeyboardHook.VKeys.CONTROL)
            {
                ctrlPressed = false;
            }
            else if (key == Config.UseHotKeyKey)
            {
                keyPressed = false;
            }
        }

        private void Hook_KeyDown(KeyboardHook.VKeys key)
        {
            if (key == KeyboardHook.VKeys.LSHIFT || key == KeyboardHook.VKeys.RSHIFT || key == KeyboardHook.VKeys.SHIFT)
            {
                shiftPressed = true;
            }else if(key == KeyboardHook.VKeys.LCONTROL || key == KeyboardHook.VKeys.RCONTROL || key == KeyboardHook.VKeys.CONTROL)
            {
                ctrlPressed = true;
            }
            else if(key == Config.UseHotKeyKey)
            {
                keyPressed = true;
            }

            this.CheckIfPressed();
        }

        private void CheckIfPressed()
        {
            bool sOkay = false, cOkay = false, kOkay = false;

            kOkay = keyPressed;

            if (Config.UseHotKeyCtrl) { if (ctrlPressed) cOkay = true; }
            else cOkay = true;

            if (Config.UseHotKeyShift) { if (shiftPressed) sOkay = true; }
            else sOkay = true;

            if(kOkay && cOkay && sOkay)
            {
                if (editMode)
                {
                    EditMode(false);
                }

                //Toggle according to state
                ToggleWindow(state != WindowState.Maximized);
            }
        }
        #endregion
    }
}